#include <stdio.h>

#include <directfb.h>

#include "config.h"
#include "network.h"
#include "devices.h"

#define DFBCHECK(x...)                                                     \
               err = x;                                                    \
               if (err != DFB_OK) {                                        \
                    fprintf( stderr, "%s <%d>:\n\t", __FILE__, __LINE__ ); \
                    DirectFBErrorFatal( #x, err );                         \
               }

static IDirectFB        *dfb     = NULL;
static IDirectFBSurface *primary = NULL;
static IDirectFBSurface *on      = NULL;
static IDirectFBSurface *off     = NULL;
static DFBResult         err     = DFB_OK;

static int offset_x = 0;
static int offset_y = 0;

static int dummy    = 0;


/* setup_devices
This function initializes the output devices. */

int *
setup_devices (void)
{
  IDirectFBImageProvider *provider;
  DFBSurfaceDescription   dsc;
  int screen_width;
  int screen_height;

  DFBCHECK (DirectFBInit (NULL, NULL));

  /* create the super interface */
  DFBCHECK (DirectFBCreate (&dfb));
  DFBCHECK (dfb->SetCooperativeLevel (dfb, DFSCL_FULLSCREEN));

  /* try to set video mode, may fail */
  dfb->SetVideoMode (dfb, 800, 600, 16);

  /* create the primary surface */
  dsc.flags = DSDESC_CAPS;
  dsc.caps = DSCAPS_PRIMARY | DSCAPS_FLIPPING;

  DFBCHECK (dfb->CreateSurface (dfb, &dsc, &primary));
  primary->GetSize (primary, &screen_width, &screen_height);

  primary->SetColor (primary, 0x0, 0x0, 0x0, 0xFF);
  primary->FillRectangle (primary, 0, 0, screen_width, screen_height);
  primary->Flip (primary, NULL, 0);
  primary->FillRectangle (primary, 0, 0, screen_width, screen_height);

  /* load images */
  DFBCHECK (dfb->CreateImageProvider (dfb, 
                                      DATADIR "lights_on.png", &provider));
  DFBCHECK(provider->GetSurfaceDescription( provider, &dsc ));
  dsc.flags = DSDESC_WIDTH | DSDESC_HEIGHT;
  DFBCHECK(dfb->CreateSurface( dfb, &dsc, &on ));
  provider->RenderTo (provider, on);
  provider->Release (provider);

  DFBCHECK (dfb->CreateImageProvider (dfb, 
                                      DATADIR "lights_off.png", &provider));
  DFBCHECK(provider->GetSurfaceDescription( provider, &dsc ));
  dsc.flags = DSDESC_WIDTH | DSDESC_HEIGHT;
  DFBCHECK(dfb->CreateSurface( dfb, &dsc, &off ));
  provider->RenderTo (provider, off);
  provider->Release (provider);

  offset_x = (screen_width  - dsc.width)  / 2;
  offset_y = (screen_height - dsc.height) / 2;

  primary->Blit (primary, off, NULL, offset_x, offset_y);
  primary->Flip (primary, NULL, 0);
  primary->Blit (primary, off, NULL, offset_x, offset_y);  

  return &dummy;
}


/* close_devices
This function closes the output devices. */

void
close_devices (int *devices)
{
  on->Release (on);
  off->Release (off);
  primary->Release (primary);
  dfb->Release (dfb);
}


/* output_frame
This function "draws" the frame on the Windows. */

void
output_frame (const frame_t *frame, 
              const int     *devices)
{
  DFBRectangle  rect;
  int x, y;

  if (frame == NULL ||
      frame->frame_width != WIDTH || frame->frame_height != HEIGHT)
    return;

  for (y = 0; y < HEIGHT; y++)
    {
      for (x = 0; x < WIDTH; x++)
        {
          rect.x = 59 + x * 23;
          rect.y = 96 + y * 42;
          rect.w = 23;
          rect.h = 42;

          primary->Blit (primary, 
                         frame->frame_data[y][x] ? on : off, 
                         &rect, offset_x + rect.x, offset_y + rect.y);
        }
    }

  primary->Flip (primary, NULL, DSFLIP_WAITFORSYNC);
}
