#include <arpa/inet.h>
#include <netinet/in.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/time.h>

#include <fcntl.h>
#include <unistd.h>

#include <stdlib.h>
#include <stdio.h>
#include <time.h>

#include "config.h"
#include "log.h"


/* setup_log
This function opens the logfile. */

int
setup_log (void)
{
  int    logfd;
  mode_t oldumask;

  oldumask = umask (S_IXUSR|S_IRWXG|S_IRWXO);
  logfd = open (LOGNAME, O_RDWR|O_CREAT|O_APPEND|O_NOFOLLOW|O_LARGEFILE,      \
		S_IRUSR|S_IWUSR);
  umask (oldumask);

  return (logfd);
}


/* close_log
This function closes the logfile. */

void
close_log (const int logfd)
{
  if (logfd != -1)
    close (logfd);
}


/* log_frame
This function writes the frame-data to the logfile */

inline void
log_frame (const frame_t * frame, const int logfd,                            \
	   const unsigned int expected, const struct sockaddr_in * addr)
{
  int               k;
  char           *  string;
  struct timeval    tv;

  if (!(gettimeofday (&tv, NULL)))
    {
      string = ctime (((time_t *) (&(tv.tv_sec))));
      string[strlen (string) - 1] = '\0';
      dprintf (logfd, "[%s %li] ", string, tv.tv_usec);
      string = inet_ntoa (addr->sin_addr);
      if (expected != frame->frame_count)
	dprintf (logfd, "Received unexpected packet from %s ", string);
      else
	dprintf (logfd, "Received expected packet from %s ", string);
      dprintf (logfd, "Expected-Count: %u Real-Count: %u Width: %u "          \
	       "Height: %u Data: ", expected, frame->frame_count,             \
	       frame->frame_width, frame->frame_height);
      for (k = 0; k < (WIDTH * HEIGHT); k++)
	{
	  (((char *) frame->frame_data)[k]) ? dprintf(logfd, "1") : \
	    dprintf(logfd, "0");
	}
      dprintf(logfd, "\n");
    }
}


