#include <arpa/inet.h>
#include <netinet/in.h>

#include <unistd.h>
#include <string.h>
#include <stdio.h>

#include "config.h"
#include "network.h"
#include "devices.h"
#include "log.h"


int
main (int argc, char ** argv)
{
  frame_t               frame;
  int                   sock,                                                 \
                        logfd,                                                \
                     *  devices;
  unsigned int          current = 0,                                          \
                        expected;
  struct sockaddr_in    addr;

  if (argc == 2)
    {
      if (!(inet_aton (argv[1], &(addr.sin_addr))))
	{
	  addr.sin_addr.s_addr = INADDR_ANY;
	}
    }
  else
    {
      addr.sin_addr.s_addr = INADDR_ANY;
    }
  addr.sin_port = htons (PORT);

  if ((devices = setup_devices ()))
    {
      if (setup_socket (&sock, &addr))
	{
	  if ((logfd = setup_log ()) != -1)
	    {
	      while (1)
		{
		  if (get_frame (sock, &addr, &frame))
		    {
		      expected = (current == 0xFFFFFFFFL) ? 0 : current + 1;
		      current = frame.frame_count;
		      log_frame (&frame, logfd, expected, &addr);
		      output_frame (&frame, devices);
		    }
		}
	      close_log (logfd);
	    }
	  close_socket (&sock);
	}
      fprintf (stderr, "Failed to open socket, exiting");
      close_devices (devices);
      devices = NULL;
    }
  fprintf (stderr, "Failed to open devices, exiting");
  exit (1);
}


